class RedHat:
    INSTRUCTION = "The Red hat: a place of emotions in thinking. There may be a strong background emotion such as fear, anger, hatred, suspicion, jealousy or love. This background limits and colours all perception. The purpose of red hat thinking is to make visible this background so that its subsequent influence can be observed. You should think about feelings, emotion and intuition. Without any need to explain your reasonging. Explaining the emotion is not allowed. You can come up with new Ideas, but they need to be based on **your** emotions and feelings (How do you feel about the ideas)."
